@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
  * queuehandler.c
- * Copyright (C) John Stebbins 2008-2017 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2018 <stebbins@stebbins>
  *
  * queuehandler.c is free software.
  *
@@ -78,6 +78,47 @@ queue_list_selection_changed_cb(GtkTreeSelection *selection, signal_user_data_t
     }
 }
 
+char *
+ghb_subtitle_short_description(const GhbValue *subsource,
+                               const GhbValue *subsettings)
+{
+    GhbValue *srt;
+    char *desc = NULL;
+
+    srt = ghb_dict_get(subsettings, "SRT");
+    if (srt != NULL)
+    {
+        const gchar *code;
+        const gchar *lang;
+        const iso639_lang_t *iso;
+
+        lang = ghb_dict_get_string(srt, "Language");
+        code = ghb_dict_get_string(srt, "Codeset");
+
+        iso = lang_lookup(lang);
+        if (iso != NULL)
+        {
+            if (iso->native_name != NULL)
+                lang = iso->native_name;
+            else
+                lang = iso->eng_name;
+        }
+
+        desc = g_strdup_printf("%s (%s)(SRT)", lang, code);
+    }
+    else if (subsource == NULL)
+    {
+        desc = g_strdup(_("Foreign Audio Scan"));
+    }
+    else
+    {
+        const char * lang = ghb_dict_get_string(subsource, "Language");
+        desc = g_strdup_printf("%s", lang);
+    }
+
+    return desc;
+}
+
 static char *
 subtitle_get_track_description(const GhbValue *subsource,
                                const GhbValue *subsettings)
@@ -120,15 +161,13 @@ subtitle_get_track_description(const GhbValue *subsource,
     }
     else if (subsource == NULL)
     {
-        desc = g_strdup(_("Foreign Audio Search"));
+        desc = g_strdup(_("Foreign Audio Scan"));
     }
     else
     {
         int track         = ghb_dict_get_int(subsettings, "Track");
-        int source        = ghb_dict_get_int(subsource, "Source");
         const char * lang = ghb_dict_get_string(subsource, "Language");
-        desc = g_strdup_printf("%d - %s (%s)", track + 1,
-                                lang, hb_subsource_name(source));
+        desc = g_strdup_printf("%d - %s", track + 1, lang);
     }
 
     return desc;
@@ -166,7 +205,7 @@ add_to_queue_list(signal_user_data_t *ud, GhbValue *queueDict, GtkTreeIter *pite
     title = ghb_dict_get_int(uiDict, "title");
     start_point = ghb_dict_get_int(uiDict, "start_point");
     end_point = ghb_dict_get_int(uiDict, "end_point");
-    vol_name = ghb_dict_get_string(uiDict, "volume_label");
+    vol_name = ghb_dict_get_string(uiDict, "volume");
     dest = ghb_dict_get_string(uiDict, "destination");
     basename = g_path_get_basename(dest);
     escape = g_markup_escape_text(basename, -1);
@@ -934,11 +973,19 @@ open_queue_file(signal_user_data_t *ud)
 }
 
 G_MODULE_EXPORT void
-queue_save_clicked_cb(GtkWidget *widget, signal_user_data_t *ud)
+queue_save_action_cb(GSimpleAction *action, GVariant *param,
+                     signal_user_data_t *ud)
 {
     save_queue_file(ud);
 }
 
+G_MODULE_EXPORT void
+queue_open_action_cb(GSimpleAction *action, GVariant *param,
+                     signal_user_data_t *ud)
+{
+    open_queue_file(ud);
+}
+
 G_MODULE_EXPORT void
 queue_open_clicked_cb(GtkWidget *widget, signal_user_data_t *ud)
 {
@@ -984,6 +1031,154 @@ queue_reload_clicked_cb(GtkWidget *widget, signal_user_data_t *ud)
     }
 }
 
+gint64
+ghb_dest_free_space(GhbValue *settings)
+{
+    GFile       *gfile;
+    GFileInfo   *info;
+    guint64      size;
+    const gchar *dest     = ghb_dict_get_string(settings, "destination");
+    gchar       *destdir  = g_path_get_dirname(dest);
+    gchar       *resolved = ghb_resolve_symlink(destdir);
+
+    gfile = g_file_new_for_path(resolved);
+    info  = g_file_query_filesystem_info(gfile,
+                                G_FILE_ATTRIBUTE_FILESYSTEM_FREE, NULL, NULL);
+    if (info != NULL)
+    {
+        if (g_file_info_has_attribute(info, G_FILE_ATTRIBUTE_FILESYSTEM_FREE))
+        {
+            size = g_file_info_get_attribute_uint64(info,
+                                    G_FILE_ATTRIBUTE_FILESYSTEM_FREE);
+            return size;
+        }
+        g_object_unref(info);
+    }
+    g_object_unref(gfile);
+    g_free(resolved);
+    g_free(destdir);
+
+    return -1;
+}
+
+gint
+ghb_find_queue_job(GhbValue *queue, gint unique_id, GhbValue **job)
+{
+    GhbValue *queueDict, *uiDict;
+    gint ii, count;
+    gint job_unique_id;
+
+    if (job != NULL)
+    {
+        *job = NULL;
+    }
+    if (unique_id == 0)  // Invalid Id
+        return -1;
+
+    count = ghb_array_len(queue);
+    for (ii = 0; ii < count; ii++)
+    {
+        queueDict = ghb_array_get(queue, ii);
+        uiDict = ghb_dict_get(queueDict, "uiSettings");
+        job_unique_id = ghb_dict_get_int(uiDict, "job_unique_id");
+        if (job_unique_id == unique_id)
+        {
+            if (job != NULL)
+            {
+                *job = queueDict;
+            }
+            return ii;
+        }
+    }
+    return -1;
+}
+
+void
+ghb_low_disk_check(signal_user_data_t *ud)
+{
+    GtkWindow       *hb_window;
+    GtkWidget       *dialog;
+    GtkResponseType  response;
+    ghb_status_t     status;
+    const char      *paused_msg = "";
+    const char      *dest;
+    gint64           free_size;
+    gint64           free_limit;
+    GhbValue        *qDict;
+    GhbValue        *settings;
+
+    if (ghb_dict_get_bool(ud->globals, "SkipDiskFreeCheck") ||
+        !ghb_dict_get_bool(ud->prefs, "DiskFreeCheck"))
+    {
+        return;
+    }
+
+    ghb_get_status(&status);
+    if (status.queue.unique_id <= 0)
+    {
+        // No current job
+        return;
+    }
+    ghb_find_queue_job(ud->queue, status.queue.unique_id, &qDict);
+    if (qDict == NULL)
+    {
+        // Failed to find queue setting!
+        return;
+    }
+    settings = ghb_dict_get(qDict, "uiSettings");
+    free_size = ghb_dest_free_space(settings);
+    if (free_size < 0)
+    {
+        // Failed to read free space
+        return;
+    }
+    // limit is in MB
+    free_limit = ghb_dict_get_int(ud->prefs, "DiskFreeLimit") * 1024 * 1024;
+    if (free_size > free_limit)
+    {
+        return;
+    }
+
+    if ((status.queue.state & GHB_STATE_WORKING) &&
+        !(status.queue.state & GHB_STATE_PAUSED))
+    {
+        paused_msg = "Encoding has been paused.\n\n";
+        ghb_pause_queue();
+    }
+    dest      = ghb_dict_get_string(settings, "destination");
+    hb_window = GTK_WINDOW(GHB_WIDGET(ud->builder, "hb_window"));
+    dialog    = gtk_message_dialog_new(hb_window, GTK_DIALOG_MODAL,
+            GTK_MESSAGE_WARNING, GTK_BUTTONS_NONE,
+            _("%sThe destination filesystem is almost full: %"PRIu64"MB free.\n"
+              "Destination: %s\n"
+              "Encode may be incomplete if you proceed.\n"),
+            paused_msg, free_size / (1024 * 1024), dest); 
+    gtk_dialog_add_buttons( GTK_DIALOG(dialog),
+                           _("Resume, I've fixed the problem"), 1,
+                           _("Resume, Don't tell me again"), 2,
+                           _("Cancel Current and Stop"), 3, 
+                           NULL);
+    response = gtk_dialog_run(GTK_DIALOG(dialog));
+    gtk_widget_destroy(dialog);
+    switch ((int)response)
+    {
+        case 1:
+            ghb_resume_queue();
+            break;
+        case 2:
+            ghb_dict_set_bool(ud->globals, "SkipDiskFreeCheck", TRUE);
+            ghb_resume_queue();
+            break;
+        case 3:
+            ghb_stop_queue();
+            ud->cancel_encode = GHB_CANCEL_ALL;
+            break;
+        default:
+            ghb_resume_queue();
+            break;
+    }
+}
+
 static gboolean
 validate_settings(signal_user_data_t *ud, GhbValue *settings, gint batch)
 {
@@ -1056,45 +1251,6 @@ validate_settings(signal_user_data_t *ud, GhbValue *settings, gint batch)
         return FALSE;
     }
 #endif
-    if (!batch)
-    {
-        GFile *gfile;
-        GFileInfo *info;
-        guint64 size;
-        gchar *resolved = ghb_resolve_symlink(destdir);
-
-        gfile = g_file_new_for_path(resolved);
-        info = g_file_query_filesystem_info(gfile,
-                            G_FILE_ATTRIBUTE_FILESYSTEM_FREE, NULL, NULL);
-        if (info != NULL)
-        {
-            if (g_file_info_has_attribute(info, G_FILE_ATTRIBUTE_FILESYSTEM_FREE))
-            {
-                size = g_file_info_get_attribute_uint64(info,
-                                        G_FILE_ATTRIBUTE_FILESYSTEM_FREE);
-
-                gint64 fsize = (guint64)10 * 1024 * 1024 * 1024;
-                if (size < fsize)
-                {
-                    message = g_strdup_printf(
-                                _("Destination filesystem is almost full: %uM free\n\n"
-                                "Encode may be incomplete if you proceed.\n"),
-                                (guint)(size / (1024L*1024L)));
-                    if (!ghb_message_dialog(hb_window, GTK_MESSAGE_QUESTION,
-                                            message, _("Cancel"), _("Proceed")))
-                    {
-                        g_free(message);
-                        g_free(destdir);
-                        return FALSE;
-                    }
-                    g_free(message);
-                }
-            }
-            g_object_unref(info);
-        }
-        g_object_unref(gfile);
-        g_free(resolved);
-    }
     g_free(destdir);
     if (g_file_test(dest, G_FILE_TEST_EXISTS))
     {
@@ -1180,7 +1336,6 @@ static gboolean
 queue_add(signal_user_data_t *ud, GhbValue *settings, gint batch)
 {
     // Add settings to the queue
-    g_debug("queue_add ()");
     if (!validate_settings(ud, settings, batch))
     {
         return FALSE;
@@ -1221,9 +1376,9 @@ queue_add(signal_user_data_t *ud, GhbValue *settings, gint batch)
 }
 
 G_MODULE_EXPORT void
-queue_add_clicked_cb(GtkWidget *widget, signal_user_data_t *ud)
+queue_add_action_cb(GSimpleAction *action, GVariant *param,
+                    signal_user_data_t *ud)
 {
-    g_debug("queue_add_clicked_cb ()");
     queue_add(ud, ud->settings, 0);
     // Validation of settings may have changed audio list
     ghb_audio_list_refresh_all(ud);
@@ -1569,10 +1724,14 @@ title_dest_file_cb(GtkWidget *widget, signal_user_data_t *ud)
 
     // Check if changing the destination file name resolves
     // a file name conflict.  Enable selection if so.
-    // Disable selection if it creates a confict!!!
-    gboolean can_select;
+    // Disable selection if it creates a conflict!!!
+    gboolean selected, can_select;
+
+    widget     = find_widget(GTK_WIDGET(row), "title_selected");
+    selected   = ghb_widget_boolean(widget);
     can_select = title_multiple_can_select(ud->settings_array, index);
-    ghb_dict_set_bool(settings, "title_selected", can_select);
+
+    ghb_dict_set_bool(settings, "title_selected", selected && can_select);
     title_add_multiple_set_sensitive(GTK_WIDGET(row), can_select);
 
     g_free(dest_file);
@@ -1605,10 +1764,14 @@ title_dest_dir_cb(GtkWidget *widget, signal_user_data_t *ud)
 
     // Check if changing the destination file name resolves
     // a file name conflict.  Enable selection if so.
-    // Disable selection if it creates a confict!!!
-    gboolean can_select;
+    // Disable selection if it creates a conflict!!!
+    gboolean selected, can_select;
+
+    widget     = find_widget(GTK_WIDGET(row), "title_selected");
+    selected   = ghb_widget_boolean(widget);
     can_select = title_multiple_can_select(ud->settings_array, index);
-    ghb_dict_set_bool(settings, "title_selected", can_select);
+
+    ghb_dict_set_bool(settings, "title_selected", selected && can_select);
     title_add_multiple_set_sensitive(GTK_WIDGET(row), can_select);
 
     g_free(dest_dir);
@@ -1682,17 +1845,22 @@ GtkWidget * title_create_row(signal_user_data_t *ud)
 }
 
 G_MODULE_EXPORT void
-queue_add_multiple_clicked_cb(GtkWidget *widget, signal_user_data_t *ud)
+queue_add_all_action_cb(GSimpleAction *action, GVariant *param,
+                        signal_user_data_t *ud)
 {
-    g_debug("queue_add_multiple_clicked_cb ()");
-
     GtkListBox *list;
     GtkWidget *row;
     gint count, ii;
     int max_title_len = 0;
+    GhbValue * preset = NULL;
 
     list = GTK_LIST_BOX(GHB_WIDGET(ud->builder, "title_add_multiple_list"));
 
+    if (ghb_dict_get_bool(ud->prefs, "SyncTitleSettings"))
+    {
+        preset = ghb_settings_to_preset(ud->settings);
+    }
+
     // Set up the list of titles
     count = ghb_array_len(ud->settings_array);
     for (ii = 0; ii < count; ii++)
@@ -1712,6 +1880,11 @@ queue_add_multiple_clicked_cb(GtkWidget *widget, signal_user_data_t *ud)
         chooser = GTK_FILE_CHOOSER(find_widget(row, "title_dir"));
 
         settings = ghb_array_get(ud->settings_array, ii);
+        if (preset != NULL)
+        {
+            ghb_preset_to_settings(settings, preset);
+            ghb_set_title_settings(ud, settings);
+        }
         title_id = ghb_dict_get_int(settings, "title");
         title = ghb_lookup_title(title_id, &titleindex);
         if (title != NULL)
@@ -1826,6 +1999,7 @@ ghb_queue_remove_row(signal_user_data_t *ud, int row)
 {
     ghb_queue_remove_row_internal(ud, row);
     ghb_save_queue(ud->queue);
+    ghb_queue_buttons_grey(ud);
 }
 
 G_MODULE_EXPORT void
@@ -1840,6 +2014,7 @@ queue_delete_all_clicked_cb(GtkWidget *widget, signal_user_data_t *ud)
     }
     ghb_save_queue(ud->queue);
     ghb_update_pending(ud);
+    ghb_queue_buttons_grey(ud);
 }
 
 G_MODULE_EXPORT void
@@ -1855,7 +2030,6 @@ queue_remove_clicked_cb(GtkWidget *widget, gchar *path, signal_user_data_t *ud)
     GhbValue *queueDict, *uiDict;
     gint status;
 
-    g_debug("queue_remove_clicked_cb ()");
     treeview = GTK_TREE_VIEW(GHB_WIDGET(ud->builder, "queue_list"));
     store = gtk_tree_view_get_model(treeview);
     treepath = gtk_tree_path_new_from_string (path);
@@ -1895,6 +2069,7 @@ queue_remove_clicked_cb(GtkWidget *widget, gchar *path, signal_user_data_t *ud)
         gtk_tree_path_free (treepath);
     }
     ghb_update_pending(ud);
+    ghb_queue_buttons_grey(ud);
 }
 
 static gint
@@ -1982,7 +2157,7 @@ queue_drag_motion_cb(
         pos = GTK_TREE_VIEW_DROP_BEFORE;
     if (pos == GTK_TREE_VIEW_DROP_INTO_OR_AFTER)
         pos = GTK_TREE_VIEW_DROP_AFTER;
-    // Don't allow droping int child items
+    // Don't allow dropping int child items
     if (gtk_tree_path_get_depth(path) > 1)
     {
         gtk_tree_path_up(path);
@@ -2119,12 +2294,13 @@ queue_drag_cb(
 void
 ghb_queue_buttons_grey(signal_user_data_t *ud)
 {
-    GtkWidget *widget;
+    GtkWidget       * widget;
+    GSimpleAction   * action;
     gint queue_count;
     gint title_id, titleindex;
     const hb_title_t *title;
     gint queue_state, scan_state;
-    gboolean show_start, show_stop, paused;
+    gboolean allow_start, show_stop, allow_add, paused;
 
     queue_count = ghb_array_len(ud->queue);
     title_id = ghb_dict_get_int(ud->settings, "title");
@@ -2133,78 +2309,78 @@ ghb_queue_buttons_grey(signal_user_data_t *ud)
     queue_state = ghb_get_queue_state();
     scan_state = ghb_get_scan_state();
 
-    show_stop = queue_state &
-                (GHB_STATE_WORKING | GHB_STATE_SEARCHING |
-                 GHB_STATE_SCANNING | GHB_STATE_MUXING | GHB_STATE_PAUSED);
-    show_start = !(scan_state & GHB_STATE_SCANNING) &&
-                    (title !=NULL || queue_count > 0);
+    show_stop   = queue_state &
+                  (GHB_STATE_WORKING | GHB_STATE_SEARCHING |
+                   GHB_STATE_SCANNING | GHB_STATE_MUXING | GHB_STATE_PAUSED);
+    allow_start = !(scan_state & GHB_STATE_SCANNING) &&
+                    (title != NULL || queue_count > 0);
+    allow_add   = !(scan_state & GHB_STATE_SCANNING) && title != NULL;
 
 
     paused = queue_state & GHB_STATE_PAUSED;
 
-    widget = GHB_WIDGET(ud->builder, "queue_save");
-    gtk_widget_set_sensitive(widget, !!queue_count);
-    widget = GHB_WIDGET(ud->builder, "queue_add");
-    gtk_widget_set_sensitive(widget, show_start);
-    widget = GHB_WIDGET(ud->builder, "queue_add_menu");
-    gtk_widget_set_sensitive(widget, show_start);
-    widget = GHB_WIDGET(ud->builder, "queue_add_multiple_menu");
-    gtk_widget_set_sensitive(widget, show_start);
+    action = G_SIMPLE_ACTION(g_action_map_lookup_action(G_ACTION_MAP(ud->app),
+                                                        "queue-save"));
+    g_simple_action_set_enabled(action, !!queue_count);
+    action = G_SIMPLE_ACTION(g_action_map_lookup_action(G_ACTION_MAP(ud->app),
+                                                        "queue-add"));
+    g_simple_action_set_enabled(action, allow_add);
+    action = G_SIMPLE_ACTION(g_action_map_lookup_action(G_ACTION_MAP(ud->app),
+                                                        "queue-add-all"));
+    g_simple_action_set_enabled(action, allow_add);
+    action = G_SIMPLE_ACTION(g_action_map_lookup_action(G_ACTION_MAP(ud->app),
+                                                        "queue-start"));
+    g_simple_action_set_enabled(action, allow_start || show_stop);
+    action = G_SIMPLE_ACTION(g_action_map_lookup_action(G_ACTION_MAP(ud->app),
+                                                        "queue-pause"));
+    g_simple_action_set_enabled(action, show_stop);
 
-    widget = GHB_WIDGET (ud->builder, "queue_start1");
+    widget = GHB_WIDGET (ud->builder, "queue_start");
     if (show_stop)
     {
-        gtk_widget_set_sensitive (widget, TRUE);
         gtk_tool_button_set_icon_name(GTK_TOOL_BUTTON(widget), "hb-stop");
-        gtk_tool_button_set_label(GTK_TOOL_BUTTON(widget), _("Stop\nEncoding"));
+        gtk_tool_button_set_label(GTK_TOOL_BUTTON(widget), _("Stop"));
         gtk_tool_item_set_tooltip_text(GTK_TOOL_ITEM(widget), _("Stop Encoding"));
     }
     else
     {
-        gtk_widget_set_sensitive (widget, show_start);
         gtk_tool_button_set_icon_name(GTK_TOOL_BUTTON(widget), "hb-start");
-        gtk_tool_button_set_label(GTK_TOOL_BUTTON(widget), _("Start\nEncoding"));
+        gtk_tool_button_set_label(GTK_TOOL_BUTTON(widget), _("Start"));
         gtk_tool_item_set_tooltip_text(GTK_TOOL_ITEM(widget), _("Start Encoding"));
     }
-    widget = GHB_WIDGET (ud->builder, "queue_pause1");
+    widget = GHB_WIDGET (ud->builder, "queue_pause");
     if (paused)
     {
-        gtk_widget_set_sensitive (widget, show_stop);
         gtk_tool_button_set_icon_name(GTK_TOOL_BUTTON(widget), "hb-start");
-        gtk_tool_button_set_label(GTK_TOOL_BUTTON(widget), _("Resume\nEncoding"));
+        gtk_tool_button_set_label(GTK_TOOL_BUTTON(widget), _("Resume"));
         gtk_tool_item_set_tooltip_text(GTK_TOOL_ITEM(widget), _("Resume Encoding"));
     }
     else
     {
-        gtk_widget_set_sensitive (widget, show_stop);
         gtk_tool_button_set_icon_name(GTK_TOOL_BUTTON(widget), "hb-pause");
-        gtk_tool_button_set_label(GTK_TOOL_BUTTON(widget), _("Pause\nEncoding"));
+        gtk_tool_button_set_label(GTK_TOOL_BUTTON(widget), _("Pause"));
         gtk_tool_item_set_tooltip_text(GTK_TOOL_ITEM(widget), _("Pause Encoding"));
     }
 
     widget = GHB_WIDGET (ud->builder, "queue_start_menu");
     if (show_stop)
     {
-        gtk_widget_set_sensitive (widget, TRUE);
         gtk_menu_item_set_label(GTK_MENU_ITEM(widget), _("S_top Encoding"));
         gtk_widget_set_tooltip_text(widget, _("Stop Encoding"));
     }
     else
     {
-        gtk_widget_set_sensitive (widget, show_start);
         gtk_menu_item_set_label(GTK_MENU_ITEM(widget), _("_Start Encoding"));
         gtk_widget_set_tooltip_text(widget, _("Start Encoding"));
     }
     widget = GHB_WIDGET (ud->builder, "queue_pause_menu");
     if (paused)
     {
-        gtk_widget_set_sensitive (widget, show_start);
         gtk_menu_item_set_label(GTK_MENU_ITEM(widget), _("_Resume Encoding"));
         gtk_widget_set_tooltip_text(widget, _("Resume Encoding"));
     }
     else
     {
-        gtk_widget_set_sensitive (widget, show_stop);
         gtk_menu_item_set_label(GTK_MENU_ITEM(widget), _("_Pause Encoding"));
         gtk_widget_set_tooltip_text(widget, _("Pause Encoding"));
     }
@@ -2226,7 +2402,8 @@ queue_list_size_allocate_cb(GtkWidget *widget, GtkAllocation *allocation, GtkCel
 }
 
 G_MODULE_EXPORT void
-queue_start_clicked_cb(GtkWidget *xwidget, signal_user_data_t *ud)
+queue_start_action_cb(GSimpleAction *action, GVariant *param,
+                      signal_user_data_t *ud)
 {
     GhbValue *queueDict, *uiDict;
     gboolean running = FALSE;
@@ -2275,9 +2452,10 @@ queue_start_clicked_cb(GtkWidget *xwidget, signal_user_data_t *ud)
 }
 
 G_MODULE_EXPORT void
-queue_pause_clicked_cb(GtkWidget *xwidget, signal_user_data_t *ud)
+queue_pause_action_cb(GSimpleAction *action, GVariant *param,
+                      signal_user_data_t *ud)
 {
-    ghb_pause_queue();
+    ghb_pause_resume_queue();
 }
 
 gboolean
